/************************************************************************
 This file is part of Saper.
 Saper is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 Saper is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Saper.  If not, see <http://www.gnu.org/licenses/>.
 *************************************************************************/

/// Work Around for Ext 2.2 bug ////

Ext.form.Checkbox.override({
    setValue : function(v) {
        var checked = this.checked;
        this.checked = (v === true || v === 'true' || v == '1' || String(v).toLowerCase() == 'on');
        
        if(this.rendered){
            this.el.dom.checked = this.checked;
            this.el.dom.defaultChecked = this.checked;
            this.wrap[this.checked? 'addClass' : 'removeClass'](this.checkedCls);
        }
        
        if(checked != this.checked){
            this.fireEvent("check", this, this.checked);
            if(this.handler){
                this.handler.call(this.scope || this, this, this.checked);
            }
        }
    },

    afterRender : function(){
        Ext.form.Checkbox.superclass.afterRender.call(this);
        this.wrap[this.checked? 'addClass' : 'removeClass'](this.checkedCls);
    }
});

/// End of Work Around ///

QoDesk.Saper = Ext.extend(Ext.app.Module, {

    moduleType : 'game',
    moduleId : 'saper-win',

	ver: '0.2.3',

	winAbout: false,
	saperColumns: 10,
	saperRows: 10,
	bombs: 10,
	cellCounter: this.saperRows * this.saperColumns,
	level: 0,
    
	init : function(){
		this.launcher = {
		text: 'Saper',
		tooltip: '<b>Saper</b><br />Game like Minesweeper in Windows',
		iconCls:'saper-icon',
		shortcutIconCls: 'saper-shortcut',
		handler : this.createWindow,
		scope: this
		};
	},

	createWindow : function(){
		var desktop = this.app.getDesktop();
		var win = desktop.getWindow('saper-win');

		//this.viewScore();

		if(!win){
			this.initField();

			this.timer = 0;
			this.gameStarted = false;
			this.counter = this.bombs;

			var saperPanel = new Ext.form.FormPanel({
				id: 'saperPanel',
				tbar: [{
					id:'SaperCounter',
//					disabled: true,
					text: this.renderBomb(this.counter)
				}, {
					text: Saper.StartBtnText,
					iconCls:'saper-start',
					tooltip: Saper.StartBtnToolTip,
					scope: this,
					handler: this.startGame
				},'->',{
					id:'SaperTimer',
//					disabled: true,
					text:this.renderBomb(this.timer)
				}],
				layout: 'fit',
				items: [this.grid]
			});

			win = desktop.createWindow({
				id: 'saper-win',
				title:'Saper',
				width:this.saperColumns * 16 + 26,
				height:this.saperRows * 19 + 78,
				resizable: false,
				iconCls: 'saper-icon',
				shim:false,
				animCollapse:false,
				constrainHeader:true,
				layout: 'fit',
				minimizable: true,
				maximizable: false,
				 tools : [{
					scope: this,
					id:'gear',
					qtip:Saper.WinConfToolTip,
					handler:this.showConf
				 }, {
					scope: this,
					id:'help',
					qtip: Saper.WinAboutTitle,
					handler:this.showAbout

				 }],
				items: saperPanel
			});
		}
		win.show();
	},

	renderBomb: function(v){
		if(v < 10){
			return '00'+v;
		}
		if(v < 100){
			return '0'+v;
		}
		if(v > 999){
			return '999';
		}
		return v;
	},
    
	getTimer: function(){
		var now = new Date();
		var diff = now - this.timeStart;
		return this.renderBomb(diff);
	},

	startTime: function(){
		if(!this.gameStarted){
			return;
		}
		if(!this.timeStart){
			var startTime = new Date();
			this.timeStart = startTime.format('U');
		}
		var tmr = Ext.getCmp('SaperTimer');
		var now = new Date();
		var diff = now.format('U') - this.timeStart;
		if(diff > 999){
			return;
		}
		if(tmr){
			tmr.setText(this.renderBomb(diff));
		}
		var timerDelayTask = new Ext.util.DelayedTask(this.startTime, this);
		timerDelayTask.delay(500);
	},
    
	startGame: function(){
		var w = Ext.getCmp('saper-win');
		w.close();
		this.createWindow();
		this.counter = this.bombs;
		var cntr = Ext.getCmp('SaperCounter');
		cntr.setText(this.renderBomb(this.counter));
		var tmr = Ext.getCmp('SaperTimer');
		tmr.setText(this.renderBomb(0));
		this.gameEnd = false;
	},
    
	gameOver: function(){
		this.gameStarted = false;
		this.gameEnd = true;
		var tmr = Ext.getCmp('SaperTimer');
		var time = tmr.getText();
		
		var grid = this.grid;
		
		for(var r = 0; r < this.saperRows; r++){
			for(var c = 0; c < this.saperColumns; c++){

				var record = grid.getStore().getAt(r);  
				var fieldName = grid.getColumnModel().getDataIndex(c); 
				var data = record.get(fieldName);
				var record_f = this.ds_f.getAt(r);  
				var data_f = record_f.get(fieldName);

				if(data == '?'){
					continue;
				}

				if(data_f == 'X'){
					record.set(fieldName, data_f);
					grid.store.commitChanges();
				}
			}
		}
                Ext.Msg.alert(Saper.MsgGameOverTitle, String.format(Saper.MsgGameOverText, this.counter, tmr.getText()));
	},
	
	winner: function(){
		this.gameStarted = false;
		this.gameEnd = true;
		var tmr = Ext.getCmp('SaperTimer');
		var time = tmr.getText();
                Ext.Msg.alert(Saper.MsgGameOverWinnerTitle, String.format(Saper.MsgGameOverWinnerText, this.bombs, tmr.getText()));
	},

	onCellClick: function(grid, r, c, e) {
		var tmr = Ext.getCmp('SaperTimer');
		if(this.gameEnd && tmr.Text != '000'){
			return;
		}
		if(!this.gameStarted){
			this.gameStarted = true;
			var startTime = new Date();
			this.timeStart = startTime.format('U');
			var timerDelayTask = new Ext.util.DelayedTask(this.startTime, this);
			timerDelayTask.delay(500);
		}
		if(r < 0 || r > this.saperRows-1 || c < 0 || c > this.saperColumns-1){
			return;
		}
		var record = grid.getStore().getAt(r);  
		var fieldName = grid.getColumnModel().getDataIndex(c); 
		var data = record.get(fieldName);
		var record_f = this.ds_f.getAt(r);  
		var data_f = record_f.get(fieldName);
		if(data_f == data){
			return;
		}
		if(data == '?'){
			return;
		}
        
		record.set(fieldName, data_f);

		if(data_f == 'X'){
			this.gameOver();        	
			grid.store.commitChanges();
			return;
		}

		this.cellCounter--;

		if(data_f == '0'){
			this.checkNeighbours(grid, r-1, c-1);
			this.checkNeighbours(grid, r-1, c);
			this.checkNeighbours(grid, r-1, c+1);
			this.checkNeighbours(grid, r, c-1);
			this.checkNeighbours(grid, r, c+1);
			this.checkNeighbours(grid, r+1, c-1);
			this.checkNeighbours(grid, r+1, c);
			this.checkNeighbours(grid, r+1, c+1);
		}
		
		if(this.cellCounter === 0){
			this.winner();
			grid.store.commitChanges();
			return;
		}
		grid.store.commitChanges();
	},
    
	onCellRightClick: function(grid, r, c, e) {
		e.preventDefault();
		e.stopEvent();
		var tmr = Ext.getCmp('SaperTimer');
		if(this.gameEnd && tmr.Text != '000'){
			return;
		}
    
		var record = grid.getStore().getAt(r);  
		var fieldName = grid.getColumnModel().getDataIndex(c); 
		var data = record.get(fieldName);

		if(data == '?'){
			record.set(fieldName, "-1");
			this.counter++;
			this.cellCounter++;
		}
		else{
			if(data == '-1'){
				record.set(fieldName, "?");
				this.cellCounter--;
				if(this.counter > 0){
					this.counter--;
				}
			}
			else{
				return;
			}
		}
	
		var cntr = Ext.getCmp('SaperCounter');
		cntr.setText(this.renderBomb(this.counter));
		grid.store.commitChanges();
	},
    
	checkNeighbours: function(grid, r, c){
		if(r >= 0 && r <= this.saperRows-1 && c >= 0 && c <= this.saperColumns-1){
			this.onCellClick(grid, r, c);
		}
	},
	
	initField: function(){
		this.cellCounter = this.saperRows * this.saperColumns;
		var RecArr = [];
		var i;

		var checkColumn = [];
		for(i = 0; i < this.saperColumns; i++)
		{
			RecArr[i] = 'c' + i;
			checkColumn[i] = {
				dataIndex: 'c' + i, 
				width: 16, 
				sortable: false,
				resizable: false,
				hideable: false,
				fixed : true,
				align: 'center',
				renderer : function(v, p, record){
					p.css += ' x-grid3-check-col-td'; 
					if(v == '?'){
						return '<div class="saper-attention">&#160;</div>';
					}
					if(v == 'X'){
						return '<div class="saper-bomb">&#160;</div>';
					}
					if(v > 0){
						return '<div class="saper-number">' + v + '</div>';
					}
					if(v < 0){
						return '<div class="saper-empty">&#160;</div>';
					}
					return '';
				}
			};
		}


		var dataDS = [];
		var dataDS_f = [];
		for(i = 0; i < this.saperRows; i++){
			dataDS[i] = [];
			dataDS_f[i] = [];
			for(var j = 0; j < this.saperColumns; j++){
				dataDS[i][j] = "-1";
				dataDS_f[i][j] = 0;
			}
		}

		if(this.ds){
			this.ds = '';
		}
			this.ds = new Ext.data.SimpleStore({
				fields: RecArr, 
				data : dataDS
			});

		dataDS_f = this.getRandomArray(dataDS_f);

		if(this.ds_f){
			this.ds_f = '';
		}
			this.ds_f = new Ext.data.SimpleStore({
				fields: RecArr, 
				data : dataDS_f
			});

		var colModel = new Ext.grid.ColumnModel(checkColumn);

		this.grid = new Ext.grid.GridPanel({ 
			cm: colModel,
			height: this.saperRows * 20 + 10, 
			iconCls: 'icon-grid',
			id: 'saperGrid',
			loadMask: true,
			header: false,
			disableSelection: true,
			ds: this.ds,
			listeners:{
				scope: this,
				cellclick: this.onCellClick,
				cellcontextmenu: this.onCellRightClick 
			}
		});
	},
	
	getRandomArray: function(data){
		var i = 0;
		if(this.bombs > this.saperColumns * this.saperRows * 0.2){
			this.bombs = Math.floor(this.saperColumns * this.saperRows * 0.2);
			var custB = Ext.getCmp('saperBombs');
			if(custB){
				custB.setValue(this.bombs);
			}
		}
		var rndCol;
		var rndRow;
		do{
			rndCol = Math.floor(Math.random() * (this.saperColumns - 1));	
			rndRow = Math.floor(Math.random() * (this.saperRows - 1));	

			if(data[rndRow][rndCol] != "X"){
				data[rndRow][rndCol] = "X";
				i++;
				this.updateNeighbours(data, rndRow - 1, rndCol - 1);
				this.updateNeighbours(data, rndRow - 1, rndCol);
				this.updateNeighbours(data, rndRow - 1, rndCol + 1);
				this.updateNeighbours(data, rndRow, rndCol - 1);
				this.updateNeighbours(data, rndRow, rndCol + 1);
				this.updateNeighbours(data, rndRow + 1, rndCol - 1);
				this.updateNeighbours(data, rndRow + 1, rndCol);
				this.updateNeighbours(data, rndRow + 1, rndCol + 1);
			}
		}while(i < this.bombs);
		
		return data;
	},
	
	updateNeighbours: function(data, r, c){
		if(r >= 0 && r <= this.saperRows-1 && c >= 0 && c <= this.saperColumns-1){
			if(data[r][c] != "X"){
				data[r][c]++;
			}
		}
	},
	
	setParams: function(l, c, r, b){
		this.level = l;
		this.saperColumns = c;
		this.saperRows = r;
		this.bombs = b; 
		this.cellCounter = this.saperRows * this.saperColumns; 
		this.ds = {};
		this.ds_f = {};

		var custC = Ext.getCmp('saperColumns');
		if(custC){
			custC.setValue(this.saperColumns);
		}
		var custR = Ext.getCmp('saperRows');
		if(custR){
			custR.setValue(this.saperRows);
		}
		var custB = Ext.getCmp('saperBombs');
		if(custB){
			custB.setValue(this.bombs);
		}
	},

	
	showConf: function(){
		var FormConf = new Ext.FormPanel({
			frame: true,
			bodyStyle: 'padding:5px',
			items: [{
				xtype: 'fieldset',
				title: Saper.WinConfFieldSetTitle0,
				autoHeight: true,
				autoWidth:true,
				labelWidth: 55,
				anchor: '95%',
				items: [{
					xtype: 'radio',
					id: 'saper-level-0',
					boxLabel: Saper.WinConfFieldSetLabel0,
					checked: (this.level === 0) ? true : false,
					hideLabel: true,
					listeners: {
						scope: this,
						check: function(field, checked){
							if(checked && this.level !== 0){
								this.setParams(0, 10, 10, 10);

								var l1 = Ext.getCmp('saper-level-1');
								l1.setValue(false);
								var l2 = Ext.getCmp('saper-level-2');
								l2.setValue(false);

								var w = Ext.getCmp('saper-win');
								w.close();
								this.createWindow();
							}
						}
					}
				},{
					xtype: 'radio',
					id: 'saper-level-1',
					boxLabel: Saper.WinConfFieldSetLabel1,
					checked: (this.level === 1) ? true : false,
					hideLabel: true,
					listeners: {
						scope: this,
						check: function(field, checked){
							if(checked && this.level !== 1){
								this.setParams(1, 15, 15, 30);

								var l0 = Ext.getCmp('saper-level-0');
								l0.setValue(false);
								var l2 = Ext.getCmp('saper-level-2');
								l2.setValue(false);

								var w = Ext.getCmp('saper-win');
								w.close();
								this.createWindow();
							}
						}
					}
				},{
					xtype: 'radio',
					id: 'saper-level-2',
					boxLabel: Saper.WinConfFieldSetLabel2,
					checked: (this.level === 2) ? true : false,
					hideLabel: true,
					listeners: {
						scope: this,
						check: function(field, checked){
							if(checked && this.level !== 2){
								this.setParams(2, 30, 20, 100);

								var l0 = Ext.getCmp('saper-level-0');
								l0.setValue(false);
								var l1 = Ext.getCmp('saper-level-1');
								l1.setValue(false);

								var w = Ext.getCmp('saper-win');
								w.close();
								this.createWindow();
							}
						}
					}
				}]
			},{
            xtype:'fieldset',
            id: 'saper-custom-config',
            checkboxToggle:true,
				title: Saper.WinConfFieldSetTitle1,
            autoHeight:true,
            defaultType: 'textfield',
            collapsed: (this.level === 3) ? false : true,
            items :[{
					fieldLabel: Saper.WinConfFieldSet1Label0,
					id: 'saperColumns',
					value: this.saperColumns,
					allowBlank:false,
					listeners: {
             		scope: this,
             		change: function(field, newValue, oldValue){
             			if(newValue < 5){
             				field.setValue(5);
             			}
             			if(newValue > 60){
             				field.setValue(60);
             			}
             		}
					}
				},{
					fieldLabel: Saper.WinConfFieldSet1Label1,
					id: 'saperRows',
					value: this.saperRows,
					allowBlank:false,
					listeners: {
             		scope: this,
             		change: function(field, newValue, oldValue){
             			if(newValue < 5){
             				field.setValue(5);
             			}
             			if(newValue > 30){
             				field.setValue(30);
             			}
             		}
					}
				},{
					fieldLabel: Saper.WinConfFieldSet1Label2,
					id: 'saperBombs',
					value: this.bombs,
					allowBlank:false,
					listeners: {
             		scope: this,
             		change: function(field, newValue, oldValue){
							var custC = Ext.getCmp('saperColumns');
							var c = 10;
							if(custC){
								c = custC.getValue();
							}
							var custR = Ext.getCmp('saperRows');
							var r = 10;
							if(custR){
								r = custR.getValue();
							}
             			if(newValue < 5){
             				field.setValue(5);
             			}
             			if(newValue > Math.floor(c * r * 0.2)){
             				field.setValue(Math.floor(c * r * 0.2));
             			}
             		}
					}
				}],
				buttons: [{
					text: Saper.WinConfApply,
					scope: this,
					handler: function(){
						var custC = Ext.getCmp('saperColumns');
						var c = 10;
						if(custC){
							c = custC.getValue();
						}
						var custR = Ext.getCmp('saperRows');
						var r = 10;
						if(custR){
							r = custR.getValue();
						}
						var custB = Ext.getCmp('saperBombs');
						var b = 10;
						if(custB){
							b = custB.getValue();
						}
						this.setParams(3, c, r, b);
						var l0 = Ext.getCmp('saper-level-0');
						l0.setValue(false);
						var l1 = Ext.getCmp('saper-level-1');
						l1.setValue(false);
						var l2 = Ext.getCmp('saper-level-2');
						l2.setValue(false);

						var w = Ext.getCmp('saper-win');
						w.close();
						this.createWindow();
					}
				}],
				listeners: {
					scope: this,
					expand: function(panel){
						var l0 = Ext.getCmp('saper-level-0');
						l0.setValue(false);
						var l1 = Ext.getCmp('saper-level-1');
						l1.setValue(false);
						var l2 = Ext.getCmp('saper-level-2');
						l2.setValue(false);
					},
					collapse: function(panel){
						var l0 = Ext.getCmp('saper-level-0');
						l0.setValue(true);
						var l1 = Ext.getCmp('saper-level-1');
						l1.setValue(false);
						var l2 = Ext.getCmp('saper-level-2');
						l2.setValue(false);
					}
				}
        }]
		});



		var winConf = new Ext.Window({
			scope: this,
			animateEl: 'mod-feed-btn',
			title: Saper.WinConfTitle,
			layout: 'fit',
			modal: true,
			iconCls: 'saper-icon-conf',
			width: 450,
			height: 400,
			bodyStyle: 'padding:5px',
			plain: true,
			items: FormConf,
			buttons: [{
				text: Saper.WinConfClose,
				scope: this,
				handler: function(){
					winConf.hide();
				}
			}]
		});
		winConf.show(this);
	}, 

	showAbout: function(){
		if(!this.winAbout) {
			this.winAbout = new Ext.Window({
				layout: 'fit',
				width: 515,
				height: 350,
				iconCls: 'saper-icon-about',
				modal: true,
				title: Saper.WinAboutTitle,
				closeAction: 'hide',
				plain: true,
				bodyStyle: 'color:#000',
				items: new Ext.TabPanel({
					autoTabs: true,
					activeTab: 0,
					border: false,
					defaults: {
						autoScroll: true
					},
					items: [{
							title: Saper.WinAboutTab0,
							html: '<div id="saper-about"><img src="system/modules/Saper/img/saper_logo.png" alt="Saper" /></div><div id="saper-about-info">Saper ver ' + this.ver + ' (c) 2008, Andriy Levytskyy - <a href="mailto:andriy.levytskyy@gmail.com">andriy.levytskyy@gmail.com</a></div>'
						}, {
							title: Saper.WinAboutTab1,
							bodyStyle: 'padding:5px',
							html: '<div id="saper-credit"><ul>' + 
										'<li><a href="http://extjs.com">ExtJs Team</a><br><span class="saper-credit-info">' + Saper.WinAboutTab1Credits0 + '</span></li>' + 
										'<li><a href="http:// famfamfam.com">famfamfam.com</a><br><span class="saper-credit-info">' + Saper.WinAboutTab1Credits1 + '</span></li>' + 
										'<li>Yannick Torr&egrave;s - <a href="mailto:yannick.torres@keliglia.com">yannick.torres@keliglia.com</a> <br><span class="saper-credit-info">' + Saper.WinAboutTab1Credits2 + '</span></li>' + 
										'<li><a href="http://extjs.com/forum/member.php?u=12208">Franck Carrouget</a><br><span class="saper-credit-info">' + Saper.WinAboutTab1Credits3 + '</span></li>' + 
										'<li><a href="http://extjs.com/forum/member.php?u=17060">drony</a><br><span class="saper-credit-info">' + Saper.WinAboutTab1Credits4 + '</span></li>' + 
										'<li><a href="http://extjs.com/forum/member.php?u=23358">qbase</a><br><span class="saper-credit-info">' + Saper.WinAboutTab1Credits5 + '</span></li>' + 
								'</ul></div>'
						}, {
							title: Saper.WinAboutTab2,
							autoLoad: {
								url: 'system/modules/Saper/license_LGPL_v2.txt'
						}
					}]
				}),
				buttons: [{
					scope:this,
					text: Saper.WinAboutClose,
					handler: function(){
						this.winAbout.hide();
					}
				}]
			});
		}
		this.winAbout.show(this);
	}
	
});
